package net.peachjean.pegdown.uml;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Map;

import net.sourceforge.plantuml.FileFormat;
import net.sourceforge.plantuml.FileFormatOption;
import net.sourceforge.plantuml.SourceStringReader;

import org.parboiled.common.Base64;
import org.pegdown.Printer;
import org.pegdown.ast.VerbatimNode;

/**
 * Description of file content.
 *
 * @author jbunting
 *         4/12/13
 */
public class PlantUMLVerbatimSerializer implements VerbatimSerializer
{
	@Override
	public void serialize(final VerbatimNode node, final Printer printer)
	{
		Type type = Type.valueOf(node.getType());

		String formatted = type.wrap(node.getText());
		final ByteArrayOutputStream baos = new ByteArrayOutputStream();
		SourceStringReader reader = new SourceStringReader(formatted);
		String desc;
		try
		{
			desc = reader.generateImage(baos, type.getFormatOption());
//			if(desc != null) {
//				throw new RuntimeException("Could not generate uml for node " + node + ". " + desc);
//			}
		}
		catch (IOException e)
		{
			throw new RuntimeException("Could not generate uml for node " + node, e);
		}
		final String rendered = type.render(baos.toByteArray(), desc);
		printer.print(rendered);
	}

	public static void addToMap(final Map<String, VerbatimSerializer> serializerMap)
	{
		PlantUMLVerbatimSerializer serializer = new PlantUMLVerbatimSerializer();
		for(Type type: Type.values())
		{
			serializerMap.put(type.name(), serializer);
		}
	}

	static enum Type
	{
		uml(OutputType.svg),
		dot(OutputType.svg),
		jcckit(OutputType.png),
		ditaa(OutputType.png);

		private final OutputType outputType;

		Type(final OutputType outputType)
		{
			this.outputType = outputType;
		}

		public String wrap(final String originalText)
		{
			return "@start" + this.toString() + "\n" + originalText + "@end" + this.toString() + "\n";
		}

		public FileFormatOption getFormatOption()
		{
			return this.outputType.getFormatOption();
		}

		public String render(byte[] bytes, final String desc)
		{
			return this.outputType.render(bytes, desc);
		}
	}

	static enum OutputType
	{
		svg
				{
					public FileFormatOption getFormatOption()
					{
						return new FileFormatOption(FileFormat.SVG);
					}

					public String render(byte[] bytes, final String desc)
					{
						return new String(bytes);
					}
				}, png
			{
				@Override
				public FileFormatOption getFormatOption()
				{
					return new FileFormatOption(FileFormat.PNG);
				}

				@Override
				public String render(final byte[] bytes, final String desc)
				{
					return String.format("<img alt=\"%s\" src=\"data:image/png;base64,%s\"/>", desc, Base64.rfc2045().encodeToString(bytes, false));
				}
			};

		public abstract FileFormatOption getFormatOption();

		public abstract String render(byte[] bytes, final String desc);
	}
}
